@@ -42,7 +42,18 @@ namespace IKVM.Reflection.Emit
 		private Type baseType;
 		private GenericParameterAttributes attr;
 
-		internal GenericTypeParameterBuilder(string name, TypeBuilder type, MethodBuilder method, int position)
+		internal GenericTypeParameterBuilder(string name, TypeBuilder type, int position)
+			: this(name, type, null, position, Signature.ELEMENT_TYPE_VAR)
+		{
+		}
+
+		internal GenericTypeParameterBuilder(string name, MethodBuilder method, int position)
+			: this(name, null, method, position, Signature.ELEMENT_TYPE_MVAR)
+		{
+		}
+
+		private GenericTypeParameterBuilder(string name, TypeBuilder type, MethodBuilder method, int position, byte sigElementType)
+			: base(sigElementType)
 		{
 			this.name = name;
 			this.type = type;
@@ -111,11 +122,6 @@ namespace IKVM.Reflection.Emit
 			get { return ModuleBuilder; }
 		}
 
-		public override bool IsGenericParameter
-		{
-			get { return true; }
-		}
-
 		public override int GenericParameterPosition
 		{
 			get { return position; }
@@ -277,7 +283,7 @@ namespace IKVM.Reflection.Emit
 			this.name = name;
 			this.typeNameSpace = ns == null ? 0 : this.ModuleBuilder.Strings.Add(ns);
 			this.typeName = this.ModuleBuilder.Strings.Add(name);
-			MarkEnumOrValueType(ns, name);
+			MarkKnownType(ns, name);
 		}
 
 		public ConstructorBuilder DefineDefaultConstructor(MethodAttributes attributes)
@@ -611,31 +617,26 @@ namespace IKVM.Reflection.Emit
 
 		public void SetCustomAttribute(CustomAttributeBuilder customBuilder)
 		{
-			Universe u = this.ModuleBuilder.universe;
-			Type type = customBuilder.Constructor.DeclaringType;
-			if (type == u.System_Runtime_InteropServices_StructLayoutAttribute)
+			switch (customBuilder.KnownCA)
 			{
-				SetStructLayoutPseudoCustomAttribute(customBuilder.DecodeBlob(this.Assembly));
-			}
-			else if (type == u.System_SerializableAttribute)
-			{
-				attribs |= TypeAttributes.Serializable;
-			}
-			else if (type == u.System_Runtime_InteropServices_ComImportAttribute)
-			{
-				attribs |= TypeAttributes.Import;
-			}
-			else if (type == u.System_Runtime_CompilerServices_SpecialNameAttribute)
-			{
-				attribs |= TypeAttributes.SpecialName;
-			}
-			else
-			{
-				if (type == u.System_Security_SuppressUnmanagedCodeSecurityAttribute)
-				{
+				case KnownCA.StructLayoutAttribute:
+					SetStructLayoutPseudoCustomAttribute(customBuilder.DecodeBlob(this.Assembly));
+					break;
+				case KnownCA.SerializableAttribute:
+					attribs |= TypeAttributes.Serializable;
+					break;
+				case KnownCA.ComImportAttribute:
+					attribs |= TypeAttributes.Import;
+					break;
+				case KnownCA.SpecialNameAttribute:
+					attribs |= TypeAttributes.SpecialName;
+					break;
+				case KnownCA.SuppressUnmanagedCodeSecurityAttribute:
 					attribs |= TypeAttributes.HasSecurity;
-				}
-				this.ModuleBuilder.SetCustomAttribute(token, customBuilder);
+					goto default;
+				default:
+					this.ModuleBuilder.SetCustomAttribute(token, customBuilder);
+					break;
 			}
 		}
 
@@ -663,7 +664,7 @@ namespace IKVM.Reflection.Emit
 			gtpb = new GenericTypeParameterBuilder[names.Length];
 			for (int i = 0; i < names.Length; i++)
 			{
-				gtpb[i] = new GenericTypeParameterBuilder(names[i], this, null, i);
+				gtpb[i] = new GenericTypeParameterBuilder(names[i], this, i);
 			}
 			return (GenericTypeParameterBuilder[])gtpb.Clone();
 		}
@@ -810,14 +811,9 @@ namespace IKVM.Reflection.Emit
 			}
 		}
 
-		public override string __Name
+		internal override TypeName TypeName
 		{
-			get { return name; }
-		}
-
-		public override string __Namespace
-		{
-			get { return ns; }
+			get { return new TypeName(ns, name); }
 		}
 
 		public override string Name
@@ -1088,14 +1084,9 @@ namespace IKVM.Reflection.Emit
 			get { return underlyingType.BaseType; }
 		}
 
-		public override string __Name
+		internal override TypeName TypeName
 		{
-			get { return underlyingType.__Name; }
-		}
-
-		public override string __Namespace
-		{
-			get { return underlyingType.__Namespace; }
+			get { return underlyingType.TypeName; }
 		}
 
 		public override string Name
