@@ -115,6 +115,12 @@ namespace IKVM.Reflection
 	 *      - Module.__ReadDataFromRVA()
 	 *      - MethodBase.GetMethodBody()
 	 *      - FieldInfo.__GetDataFromRVA()
+	 *
+	 *   DeterministicOutput
+	 *      The generated output file will depend only on the input. In other words,
+	 *      the PE file header time stamp will be set to zero and the module version
+	 *      id will be based on a SHA1 of the contents, instead of a random guid.
+	 *      This option can not be used in combination with PDB file generation.
 	 */
 
 	[Flags]
@@ -127,6 +133,9 @@ namespace IKVM.Reflection
 		DontProvideAutomaticDefaultConstructor = 8,
 		MetadataOnly = 16,
 		ResolveMissingMembers = 32,
+		DisableWindowsRuntimeProjection = 64,
+		DecodeVersionInfoAttributeBlobs = 128,
+		DeterministicOutput = 256,
 	}
 
 	public sealed class Universe : IDisposable
@@ -169,26 +178,16 @@ namespace IKVM.Reflection
 		private Type typeof_System_DateTime;
 		private Type typeof_System_DBNull;
 		private Type typeof_System_Decimal;
-		private Type typeof_System_NonSerializedAttribute;
-		private Type typeof_System_SerializableAttribute;
 		private Type typeof_System_AttributeUsageAttribute;
 		private Type typeof_System_Runtime_InteropServices_DllImportAttribute;
 		private Type typeof_System_Runtime_InteropServices_FieldOffsetAttribute;
-		private Type typeof_System_Runtime_InteropServices_InAttribute;
 		private Type typeof_System_Runtime_InteropServices_MarshalAsAttribute;
 		private Type typeof_System_Runtime_InteropServices_UnmanagedType;
 		private Type typeof_System_Runtime_InteropServices_VarEnum;
-		private Type typeof_System_Runtime_InteropServices_OutAttribute;
-		private Type typeof_System_Runtime_InteropServices_StructLayoutAttribute;
-		private Type typeof_System_Runtime_InteropServices_OptionalAttribute;
 		private Type typeof_System_Runtime_InteropServices_PreserveSigAttribute;
 		private Type typeof_System_Runtime_InteropServices_CallingConvention;
 		private Type typeof_System_Runtime_InteropServices_CharSet;
-		private Type typeof_System_Runtime_InteropServices_ComImportAttribute;
 		private Type typeof_System_Runtime_CompilerServices_DecimalConstantAttribute;
-		private Type typeof_System_Runtime_CompilerServices_SpecialNameAttribute;
-		private Type typeof_System_Runtime_CompilerServices_MethodImplAttribute;
-		private Type typeof_System_Security_SuppressUnmanagedCodeSecurityAttribute;
 		private Type typeof_System_Reflection_AssemblyCopyrightAttribute;
 		private Type typeof_System_Reflection_AssemblyTrademarkAttribute;
 		private Type typeof_System_Reflection_AssemblyProductAttribute;
@@ -237,17 +236,17 @@ namespace IKVM.Reflection
 			get { return Load("mscorlib"); }
 		}
 
-		private Type ImportMscorlibType(System.Type type)
+		private Type ImportMscorlibType(string ns, string name)
 		{
 			if (Mscorlib.__IsMissing)
 			{
-				return Mscorlib.ResolveType(null, new TypeName(type.Namespace, type.Name));
+				return Mscorlib.ResolveType(null, new TypeName(ns, name));
 			}
 			// We use FindType instead of ResolveType here, because on some versions of mscorlib some of
 			// the special types we use/support are missing and the type properties are defined to
 			// return null in that case.
 			// Note that we don't have to unescape type.Name here, because none of the names contain special characters.
-			return Mscorlib.FindType(new TypeName(type.Namespace, type.Name));
+			return Mscorlib.FindType(new TypeName(ns, name));
 		}
 
 		private Type ResolvePrimitive(string name)
@@ -375,172 +374,122 @@ namespace IKVM.Reflection
 
 		internal Type System_DateTime
 		{
-			get { return typeof_System_DateTime ?? (typeof_System_DateTime = ImportMscorlibType(typeof(System.DateTime))); }
+			get { return typeof_System_DateTime ?? (typeof_System_DateTime = ImportMscorlibType("System", "DateTime")); }
 		}
 
 		internal Type System_DBNull
 		{
-			get { return typeof_System_DBNull ?? (typeof_System_DBNull = ImportMscorlibType(typeof(System.DBNull))); }
+			get { return typeof_System_DBNull ?? (typeof_System_DBNull = ImportMscorlibType("System", "DBNull")); }
 		}
 
 		internal Type System_Decimal
 		{
-			get { return typeof_System_Decimal ?? (typeof_System_Decimal = ImportMscorlibType(typeof(System.Decimal))); }
-		}
-
-		internal Type System_NonSerializedAttribute
-		{
-			get { return typeof_System_NonSerializedAttribute ?? (typeof_System_NonSerializedAttribute = ImportMscorlibType(typeof(System.NonSerializedAttribute))); }
-		}
-
-		internal Type System_SerializableAttribute
-		{
-			get { return typeof_System_SerializableAttribute ?? (typeof_System_SerializableAttribute = ImportMscorlibType(typeof(System.SerializableAttribute))); }
+			get { return typeof_System_Decimal ?? (typeof_System_Decimal = ImportMscorlibType("System", "Decimal")); }
 		}
 
 		internal Type System_AttributeUsageAttribute
 		{
-			get { return typeof_System_AttributeUsageAttribute ?? (typeof_System_AttributeUsageAttribute = ImportMscorlibType(typeof(System.AttributeUsageAttribute))); }
+			get { return typeof_System_AttributeUsageAttribute ?? (typeof_System_AttributeUsageAttribute = ImportMscorlibType("System", "AttributeUsageAttribute")); }
 		}
 
 		internal Type System_Runtime_InteropServices_DllImportAttribute
 		{
-			get { return typeof_System_Runtime_InteropServices_DllImportAttribute ?? (typeof_System_Runtime_InteropServices_DllImportAttribute = ImportMscorlibType(typeof(System.Runtime.InteropServices.DllImportAttribute))); }
+			get { return typeof_System_Runtime_InteropServices_DllImportAttribute ?? (typeof_System_Runtime_InteropServices_DllImportAttribute = ImportMscorlibType("System.Runtime.InteropServices", "DllImportAttribute")); }
 		}
 
 		internal Type System_Runtime_InteropServices_FieldOffsetAttribute
 		{
-			get { return typeof_System_Runtime_InteropServices_FieldOffsetAttribute ?? (typeof_System_Runtime_InteropServices_FieldOffsetAttribute = ImportMscorlibType(typeof(System.Runtime.InteropServices.FieldOffsetAttribute))); }
-		}
-
-		internal Type System_Runtime_InteropServices_InAttribute
-		{
-			get { return typeof_System_Runtime_InteropServices_InAttribute ?? (typeof_System_Runtime_InteropServices_InAttribute = ImportMscorlibType(typeof(System.Runtime.InteropServices.InAttribute))); }
+			get { return typeof_System_Runtime_InteropServices_FieldOffsetAttribute ?? (typeof_System_Runtime_InteropServices_FieldOffsetAttribute = ImportMscorlibType("System.Runtime.InteropServices", "FieldOffsetAttribute")); }
 		}
 
 		internal Type System_Runtime_InteropServices_MarshalAsAttribute
 		{
-			get { return typeof_System_Runtime_InteropServices_MarshalAsAttribute ?? (typeof_System_Runtime_InteropServices_MarshalAsAttribute = ImportMscorlibType(typeof(System.Runtime.InteropServices.MarshalAsAttribute))); }
+			get { return typeof_System_Runtime_InteropServices_MarshalAsAttribute ?? (typeof_System_Runtime_InteropServices_MarshalAsAttribute = ImportMscorlibType("System.Runtime.InteropServices", "MarshalAsAttribute")); }
 		}
 
 		internal Type System_Runtime_InteropServices_UnmanagedType
 		{
-			get { return typeof_System_Runtime_InteropServices_UnmanagedType ?? (typeof_System_Runtime_InteropServices_UnmanagedType = ImportMscorlibType(typeof(System.Runtime.InteropServices.UnmanagedType))); }
+			get { return typeof_System_Runtime_InteropServices_UnmanagedType ?? (typeof_System_Runtime_InteropServices_UnmanagedType = ImportMscorlibType("System.Runtime.InteropServices", "UnmanagedType")); }
 		}
 
 		internal Type System_Runtime_InteropServices_VarEnum
 		{
-			get { return typeof_System_Runtime_InteropServices_VarEnum ?? (typeof_System_Runtime_InteropServices_VarEnum = ImportMscorlibType(typeof(System.Runtime.InteropServices.VarEnum))); }
-		}
-
-		internal Type System_Runtime_InteropServices_OutAttribute
-		{
-			get { return typeof_System_Runtime_InteropServices_OutAttribute ?? (typeof_System_Runtime_InteropServices_OutAttribute = ImportMscorlibType(typeof(System.Runtime.InteropServices.OutAttribute))); }
-		}
-
-		internal Type System_Runtime_InteropServices_StructLayoutAttribute
-		{
-			get { return typeof_System_Runtime_InteropServices_StructLayoutAttribute ?? (typeof_System_Runtime_InteropServices_StructLayoutAttribute = ImportMscorlibType(typeof(System.Runtime.InteropServices.StructLayoutAttribute))); }
-		}
-
-		internal Type System_Runtime_InteropServices_OptionalAttribute
-		{
-			get { return typeof_System_Runtime_InteropServices_OptionalAttribute ?? (typeof_System_Runtime_InteropServices_OptionalAttribute = ImportMscorlibType(typeof(System.Runtime.InteropServices.OptionalAttribute))); }
+			get { return typeof_System_Runtime_InteropServices_VarEnum ?? (typeof_System_Runtime_InteropServices_VarEnum = ImportMscorlibType("System.Runtime.InteropServices", "VarEnum")); }
 		}
 
 		internal Type System_Runtime_InteropServices_PreserveSigAttribute
 		{
-			get { return typeof_System_Runtime_InteropServices_PreserveSigAttribute ?? (typeof_System_Runtime_InteropServices_PreserveSigAttribute = ImportMscorlibType(typeof(System.Runtime.InteropServices.PreserveSigAttribute))); }
+			get { return typeof_System_Runtime_InteropServices_PreserveSigAttribute ?? (typeof_System_Runtime_InteropServices_PreserveSigAttribute = ImportMscorlibType("System.Runtime.InteropServices", "PreserveSigAttribute")); }
 		}
 
 		internal Type System_Runtime_InteropServices_CallingConvention
 		{
-			get { return typeof_System_Runtime_InteropServices_CallingConvention ?? (typeof_System_Runtime_InteropServices_CallingConvention = ImportMscorlibType(typeof(System.Runtime.InteropServices.CallingConvention))); }
+			get { return typeof_System_Runtime_InteropServices_CallingConvention ?? (typeof_System_Runtime_InteropServices_CallingConvention = ImportMscorlibType("System.Runtime.InteropServices", "CallingConvention")); }
 		}
 
 		internal Type System_Runtime_InteropServices_CharSet
 		{
-			get { return typeof_System_Runtime_InteropServices_CharSet ?? (typeof_System_Runtime_InteropServices_CharSet = ImportMscorlibType(typeof(System.Runtime.InteropServices.CharSet))); }
-		}
-
-		internal Type System_Runtime_InteropServices_ComImportAttribute
-		{
-			get { return typeof_System_Runtime_InteropServices_ComImportAttribute ?? (typeof_System_Runtime_InteropServices_ComImportAttribute = ImportMscorlibType(typeof(System.Runtime.InteropServices.ComImportAttribute))); }
+			get { return typeof_System_Runtime_InteropServices_CharSet ?? (typeof_System_Runtime_InteropServices_CharSet = ImportMscorlibType("System.Runtime.InteropServices", "CharSet")); }
 		}
 
 		internal Type System_Runtime_CompilerServices_DecimalConstantAttribute
 		{
-			get { return typeof_System_Runtime_CompilerServices_DecimalConstantAttribute ?? (typeof_System_Runtime_CompilerServices_DecimalConstantAttribute = ImportMscorlibType(typeof(System.Runtime.CompilerServices.DecimalConstantAttribute))); }
-		}
-
-		internal Type System_Runtime_CompilerServices_SpecialNameAttribute
-		{
-			get { return typeof_System_Runtime_CompilerServices_SpecialNameAttribute ?? (typeof_System_Runtime_CompilerServices_SpecialNameAttribute = ImportMscorlibType(typeof(System.Runtime.CompilerServices.SpecialNameAttribute))); }
-		}
-
-		internal Type System_Runtime_CompilerServices_MethodImplAttribute
-		{
-			get { return typeof_System_Runtime_CompilerServices_MethodImplAttribute ?? (typeof_System_Runtime_CompilerServices_MethodImplAttribute = ImportMscorlibType(typeof(System.Runtime.CompilerServices.MethodImplAttribute))); }
-		}
-
-		internal Type System_Security_SuppressUnmanagedCodeSecurityAttribute
-		{
-			get { return typeof_System_Security_SuppressUnmanagedCodeSecurityAttribute ?? (typeof_System_Security_SuppressUnmanagedCodeSecurityAttribute = ImportMscorlibType(typeof(System.Security.SuppressUnmanagedCodeSecurityAttribute))); }
+			get { return typeof_System_Runtime_CompilerServices_DecimalConstantAttribute ?? (typeof_System_Runtime_CompilerServices_DecimalConstantAttribute = ImportMscorlibType("System.Runtime.CompilerServices", "DecimalConstantAttribute")); }
 		}
 
 		internal Type System_Reflection_AssemblyCopyrightAttribute
 		{
-			get { return typeof_System_Reflection_AssemblyCopyrightAttribute ?? (typeof_System_Reflection_AssemblyCopyrightAttribute = ImportMscorlibType(typeof(System.Reflection.AssemblyCopyrightAttribute))); }
+			get { return typeof_System_Reflection_AssemblyCopyrightAttribute ?? (typeof_System_Reflection_AssemblyCopyrightAttribute = ImportMscorlibType("System.Reflection", "AssemblyCopyrightAttribute")); }
 		}
 
 		internal Type System_Reflection_AssemblyTrademarkAttribute
 		{
-			get { return typeof_System_Reflection_AssemblyTrademarkAttribute ?? (typeof_System_Reflection_AssemblyTrademarkAttribute = ImportMscorlibType(typeof(System.Reflection.AssemblyTrademarkAttribute))); }
+			get { return typeof_System_Reflection_AssemblyTrademarkAttribute ?? (typeof_System_Reflection_AssemblyTrademarkAttribute = ImportMscorlibType("System.Reflection", "AssemblyTrademarkAttribute")); }
 		}
 
 		internal Type System_Reflection_AssemblyProductAttribute
 		{
-			get { return typeof_System_Reflection_AssemblyProductAttribute ?? (typeof_System_Reflection_AssemblyProductAttribute = ImportMscorlibType(typeof(System.Reflection.AssemblyProductAttribute))); }
+			get { return typeof_System_Reflection_AssemblyProductAttribute ?? (typeof_System_Reflection_AssemblyProductAttribute = ImportMscorlibType("System.Reflection", "AssemblyProductAttribute")); }
 		}
 
 		internal Type System_Reflection_AssemblyCompanyAttribute
 		{
-			get { return typeof_System_Reflection_AssemblyCompanyAttribute ?? (typeof_System_Reflection_AssemblyCompanyAttribute = ImportMscorlibType(typeof(System.Reflection.AssemblyCompanyAttribute))); }
+			get { return typeof_System_Reflection_AssemblyCompanyAttribute ?? (typeof_System_Reflection_AssemblyCompanyAttribute = ImportMscorlibType("System.Reflection", "AssemblyCompanyAttribute")); }
 		}
 
 		internal Type System_Reflection_AssemblyDescriptionAttribute
 		{
-			get { return typeof_System_Reflection_AssemblyDescriptionAttribute ?? (typeof_System_Reflection_AssemblyDescriptionAttribute = ImportMscorlibType(typeof(System.Reflection.AssemblyDescriptionAttribute))); }
+			get { return typeof_System_Reflection_AssemblyDescriptionAttribute ?? (typeof_System_Reflection_AssemblyDescriptionAttribute = ImportMscorlibType("System.Reflection", "AssemblyDescriptionAttribute")); }
 		}
 
 		internal Type System_Reflection_AssemblyTitleAttribute
 		{
-			get { return typeof_System_Reflection_AssemblyTitleAttribute ?? (typeof_System_Reflection_AssemblyTitleAttribute = ImportMscorlibType(typeof(System.Reflection.AssemblyTitleAttribute))); }
+			get { return typeof_System_Reflection_AssemblyTitleAttribute ?? (typeof_System_Reflection_AssemblyTitleAttribute = ImportMscorlibType("System.Reflection", "AssemblyTitleAttribute")); }
 		}
 
 		internal Type System_Reflection_AssemblyInformationalVersionAttribute
 		{
-			get { return typeof_System_Reflection_AssemblyInformationalVersionAttribute ?? (typeof_System_Reflection_AssemblyInformationalVersionAttribute = ImportMscorlibType(typeof(System.Reflection.AssemblyInformationalVersionAttribute))); }
+			get { return typeof_System_Reflection_AssemblyInformationalVersionAttribute ?? (typeof_System_Reflection_AssemblyInformationalVersionAttribute = ImportMscorlibType("System.Reflection", "AssemblyInformationalVersionAttribute")); }
 		}
 
 		internal Type System_Reflection_AssemblyFileVersionAttribute
 		{
-			get { return typeof_System_Reflection_AssemblyFileVersionAttribute ?? (typeof_System_Reflection_AssemblyFileVersionAttribute = ImportMscorlibType(typeof(System.Reflection.AssemblyFileVersionAttribute))); }
+			get { return typeof_System_Reflection_AssemblyFileVersionAttribute ?? (typeof_System_Reflection_AssemblyFileVersionAttribute = ImportMscorlibType("System.Reflection", "AssemblyFileVersionAttribute")); }
 		}
 
 		internal Type System_Security_Permissions_CodeAccessSecurityAttribute
 		{
-			get { return typeof_System_Security_Permissions_CodeAccessSecurityAttribute ?? (typeof_System_Security_Permissions_CodeAccessSecurityAttribute = ImportMscorlibType(typeof(System.Security.Permissions.CodeAccessSecurityAttribute))); }
+			get { return typeof_System_Security_Permissions_CodeAccessSecurityAttribute ?? (typeof_System_Security_Permissions_CodeAccessSecurityAttribute = ImportMscorlibType("System.Security.Permissions", "CodeAccessSecurityAttribute")); }
 		}
 
 		internal Type System_Security_Permissions_PermissionSetAttribute
 		{
-			get { return typeof_System_Security_Permissions_PermissionSetAttribute ?? (typeof_System_Security_Permissions_PermissionSetAttribute = ImportMscorlibType(typeof(System.Security.Permissions.PermissionSetAttribute))); }
+			get { return typeof_System_Security_Permissions_PermissionSetAttribute ?? (typeof_System_Security_Permissions_PermissionSetAttribute = ImportMscorlibType("System.Security.Permissions", "PermissionSetAttribute")); }
 		}
 
 		internal Type System_Security_Permissions_SecurityAction
 		{
-			get { return typeof_System_Security_Permissions_SecurityAction ?? (typeof_System_Security_Permissions_SecurityAction = ImportMscorlibType(typeof(System.Security.Permissions.SecurityAction))); }
+			get { return typeof_System_Security_Permissions_SecurityAction ?? (typeof_System_Security_Permissions_SecurityAction = ImportMscorlibType("System.Security.Permissions", "SecurityAction")); }
 		}
 
 		internal bool HasMscorlib
@@ -663,12 +612,22 @@ namespace IKVM.Reflection
 		}
 
 		public RawModule OpenRawModule(Stream stream, string location)
+		{
+			return OpenRawModule(stream, location, false);
+		}
+
+		public RawModule OpenMappedRawModule(Stream stream, string location)
+		{
+			return OpenRawModule(stream, location, true);
+		}
+
+		private RawModule OpenRawModule(Stream stream, string location, bool mapped)
 		{
 			if (!stream.CanRead || !stream.CanSeek || stream.Position != 0)
 			{
 				throw new ArgumentException("Stream must support read/seek and current position must be zero.", "stream");
 			}
-			return new RawModule(new ModuleReader(null, this, stream, location));
+			return new RawModule(new ModuleReader(null, this, stream, location, mapped));
 		}
 
 		public Assembly LoadAssembly(RawModule module)
@@ -723,7 +682,7 @@ namespace IKVM.Reflection
 				for (int i = 0; i < assemblies.Count; i++)
 				{
 					AssemblyComparisonResult result;
-					if (simpleName.Equals(assemblies[i].Name, StringComparison.InvariantCultureIgnoreCase)
+					if (simpleName.Equals(assemblies[i].Name, StringComparison.OrdinalIgnoreCase)
 						&& CompareAssemblyIdentity(refname, false, assemblies[i].FullName, false, out result))
 					{
 						asm = assemblies[i];
@@ -741,7 +700,7 @@ namespace IKVM.Reflection
 			foreach (AssemblyBuilder asm in dynamicAssemblies)
 			{
 				AssemblyComparisonResult result;
-				if (simpleName.Equals(asm.Name, StringComparison.InvariantCultureIgnoreCase)
+				if (simpleName.Equals(asm.Name, StringComparison.OrdinalIgnoreCase)
 					&& CompareAssemblyIdentity(refname, false, asm.FullName, false, out result))
 				{
 					return asm;
@@ -805,6 +764,9 @@ namespace IKVM.Reflection
 			{
 				return asm;
 			}
+#if CORECLR
+			return null;
+#else
 			string fileName;
 			if (throwOnError)
 			{
@@ -835,6 +797,7 @@ namespace IKVM.Reflection
 				}
 			}
 			return LoadFile(fileName);
+#endif
 		}
 
 		public Type GetType(string assemblyQualifiedTypeName)
@@ -890,6 +853,55 @@ namespace IKVM.Reflection
 			return parser.GetType(this, context == null ? null : context.ManifestModule, false, assemblyQualifiedTypeName, true, false);
 		}
 
+		public Type GetBuiltInType(string ns, string name)
+		{
+			if (ns != "System")
+			{
+				return null;
+			}
+			switch (name)
+			{
+				case "Boolean":
+					return System_Boolean;
+				case "Char":
+					return System_Char;
+				case "Object":
+					return System_Object;
+				case "String":
+					return System_String;
+				case "Single":
+					return System_Single;
+				case "Double":
+					return System_Double;
+				case "SByte":
+					return System_SByte;
+				case "Int16":
+					return System_Int16;
+				case "Int32":
+					return System_Int32;
+				case "Int64":
+					return System_Int64;
+				case "IntPtr":
+					return System_IntPtr;
+				case "UIntPtr":
+					return System_UIntPtr;
+				case "TypedReference":
+					return System_TypedReference;
+				case "Byte":
+					return System_Byte;
+				case "UInt16":
+					return System_UInt16;
+				case "UInt32":
+					return System_UInt32;
+				case "UInt64":
+					return System_UInt64;
+				case "Void":
+					return System_Void;
+				default:
+					return null;
+			}
+		}
+
 		public Assembly[] GetAssemblies()
 		{
 			Assembly[] array = new Assembly[assemblies.Count + dynamicAssemblies.Count];
@@ -907,44 +919,55 @@ namespace IKVM.Reflection
 		// this is equivalent to the Fusion CompareAssemblyIdentity API
 		public bool CompareAssemblyIdentity(string assemblyIdentity1, bool unified1, string assemblyIdentity2, bool unified2, out AssemblyComparisonResult result)
 		{
+#if CORECLR
+			return Fusion.CompareAssemblyIdentityPure(assemblyIdentity1, unified1, assemblyIdentity2, unified2, out result);
+#else
 			return useNativeFusion
 				? Fusion.CompareAssemblyIdentityNative(assemblyIdentity1, unified1, assemblyIdentity2, unified2, out result)
 				: Fusion.CompareAssemblyIdentityPure(assemblyIdentity1, unified1, assemblyIdentity2, unified2, out result);
+#endif
 		}
 
 		public AssemblyBuilder DefineDynamicAssembly(AssemblyName name, AssemblyBuilderAccess access)
 		{
-			return DefineDynamicAssemblyImpl(name, access, null, null, null, null);
+			return new AssemblyBuilder(this, name, null, null);
 		}
 
 		public AssemblyBuilder DefineDynamicAssembly(AssemblyName name, AssemblyBuilderAccess access, IEnumerable<CustomAttributeBuilder> assemblyAttributes)
 		{
-			AssemblyBuilder ab = DefineDynamicAssembly(name, access);
-			foreach (CustomAttributeBuilder cab in assemblyAttributes)
-			{
-				ab.SetCustomAttribute(cab);
-			}
-			return ab;
+			return new AssemblyBuilder(this, name, null, assemblyAttributes);
 		}
 
 		public AssemblyBuilder DefineDynamicAssembly(AssemblyName name, AssemblyBuilderAccess access, string dir)
 		{
-			return DefineDynamicAssemblyImpl(name, access, dir, null, null, null);
+			return new AssemblyBuilder(this, name, dir, null);
 		}
 
+#if !CORECLR
 #if NET_4_0
 		[Obsolete]
 #endif
 		public AssemblyBuilder DefineDynamicAssembly(AssemblyName name, AssemblyBuilderAccess access, string dir, PermissionSet requiredPermissions, PermissionSet optionalPermissions, PermissionSet refusedPermissions)
 		{
-			return DefineDynamicAssemblyImpl(name, access, dir, requiredPermissions, optionalPermissions, refusedPermissions);
+			AssemblyBuilder ab = new AssemblyBuilder(this, name, dir, null);
+			AddLegacyPermissionSet(ab, requiredPermissions, System.Security.Permissions.SecurityAction.RequestMinimum);
+			AddLegacyPermissionSet(ab, optionalPermissions, System.Security.Permissions.SecurityAction.RequestOptional);
+			AddLegacyPermissionSet(ab, refusedPermissions, System.Security.Permissions.SecurityAction.RequestRefuse);
+			return ab;
 		}
 
-		private AssemblyBuilder DefineDynamicAssemblyImpl(AssemblyName name, AssemblyBuilderAccess access, string dir, PermissionSet requiredPermissions, PermissionSet optionalPermissions, PermissionSet refusedPermissions)
+		private static void AddLegacyPermissionSet(AssemblyBuilder ab, PermissionSet permissionSet, System.Security.Permissions.SecurityAction action)
+		{
+			if (permissionSet != null)
+			{
+				ab.__AddDeclarativeSecurity(CustomAttributeBuilder.__FromBlob(CustomAttributeBuilder.LegacyPermissionSet, (int)action, Encoding.Unicode.GetBytes(permissionSet.ToXml().ToString())));
+			}
+		}
+#endif
+
+		internal void RegisterDynamicAssembly(AssemblyBuilder asm)
 		{
-			AssemblyBuilder asm = new AssemblyBuilder(this, name, dir, requiredPermissions, optionalPermissions, refusedPermissions);
 			dynamicAssemblies.Add(asm);
-			return asm;
  		}
 
 		internal void RenameAssembly(Assembly assembly, AssemblyName oldName)
@@ -1085,7 +1108,11 @@ namespace IKVM.Reflection
 				}
 				return method;
 			}
+#if CORECLR
+			throw new MissingMethodException(declaringType.ToString() + "." + name);
+#else
 			throw new MissingMethodException(declaringType.ToString(), name);
+#endif
 		}
 
 		internal FieldInfo GetMissingFieldOrThrow(Module requester, Type declaringType, string name, FieldSignature signature)
@@ -1099,7 +1126,11 @@ namespace IKVM.Reflection
 				}
 				return field;
 			}
+#if CORECLR
+			throw new MissingFieldException(declaringType.ToString() + "." + name);
+#else
 			throw new MissingFieldException(declaringType.ToString(), name);
+#endif
 		}
 
 		internal PropertyInfo GetMissingPropertyOrThrow(Module requester, Type declaringType, string name, PropertySignature propertySignature)
@@ -1115,7 +1146,11 @@ namespace IKVM.Reflection
 				}
 				return property;
 			}
+#if CORECLR
+			throw new System.MissingMemberException(declaringType.ToString() + "." + name);
+#else
 			throw new System.MissingMemberException(declaringType.ToString(), name);
+#endif
 		}
 
 		internal Type CanonicalizeType(Type type)
@@ -1167,6 +1202,11 @@ namespace IKVM.Reflection
 			}
 		}
 
+		public static Universe FromAssembly(Assembly assembly)
+		{
+			return assembly.universe;
+		}
+
 		internal bool ResolveMissingTypeIsValueType(MissingType missingType)
 		{
 			if (missingTypeIsValueType != null)
@@ -1190,5 +1230,20 @@ namespace IKVM.Reflection
 		{
 			get { return (options & UniverseOptions.MetadataOnly) != 0; }
 		}
+
+		internal bool WindowsRuntimeProjection
+		{
+			get { return (options & UniverseOptions.DisableWindowsRuntimeProjection) == 0; }
+		}
+
+		internal bool DecodeVersionInfoAttributeBlobs
+		{
+			get { return (options & UniverseOptions.DecodeVersionInfoAttributeBlobs) != 0; }
+		}
+
+		internal bool Deterministic
+		{
+			get { return (options & UniverseOptions.DeterministicOutput) != 0; }
+		}
 	}
 }
