@@ -46,7 +46,7 @@ using NUnit.Framework;
 #if NDESK_OPTIONS
 namespace Tests.NDesk.Options
 #else
-namespace Tests.Mono.Options
+namespace MonoTests.Mono.Options
 #endif
 {
 	class FooConverter : TypeConverter {
@@ -193,7 +193,7 @@ namespace Tests.Mono.Options
 			Assert.AreEqual (libs [1], null);
 
 			Utils.AssertException (typeof(OptionException), 
-					"Cannot bundle unregistered option '-V'.",
+					"Cannot use unregistered option 'V' in bundle '-EVALUENOTSUP'.",
 					p, v => { v.Parse (_("-EVALUENOTSUP")); });
 		}
 
@@ -354,8 +354,11 @@ namespace Tests.Mono.Options
 					p, v => { v.Parse (_("-a", "-b")); });
 			Assert.AreEqual (a, "-b");
 			Utils.AssertException (typeof(ArgumentNullException),
-					"Argument cannot be null.\nParameter name: option",
+					"Value cannot be null.\nParameter name: option",
 					p, v => { v.Add ((Option) null); });
+			Utils.AssertException (typeof(ArgumentNullException),
+					"Value cannot be null.\nParameter name: header",
+					p, v => { v.Add ((string) null); });
 
 			// bad type
 			Utils.AssertException (typeof(OptionException),
@@ -367,11 +370,11 @@ namespace Tests.Mono.Options
 
 			// try to bundle with an option requiring a value
 			Utils.AssertException (typeof(OptionException), 
-					"Cannot bundle unregistered option '-z'.", 
+					"Cannot use unregistered option 'z' in bundle '-cz'.",
 					p, v => { v.Parse (_("-cz", "extra")); });
 
 			Utils.AssertException (typeof(ArgumentNullException), 
-					"Argument cannot be null.\nParameter name: action",
+					"Value cannot be null.\nParameter name: action",
 					p, v => { v.Add ("foo", (Action<string>) null); });
 			Utils.AssertException (typeof(ArgumentException), 
 					"Cannot provide maxValueCount of 2 for OptionValueType.None.\nParameter name: maxValueCount",
@@ -382,6 +385,9 @@ namespace Tests.Mono.Options
 		public void WriteOptionDescriptions ()
 		{
 			var p = new OptionSet () {
+				"\n:Category 1:",
+				{ "hidden",             "hidden option, invisible in help",     v => {}, true },
+				{ "hidden2=",           "hidden option, invisible in help",     (k, v) => {}, true },
 				{ "p|indicator-style=", "append / indicator to directories",    v => {} },
 				{ "color:",             "controls color info",                  v => {} },
 				{ "color2:",            "set {color}",                          v => {} },
@@ -396,7 +402,7 @@ namespace Tests.Mono.Options
 						"  item 2",
 					v => {} },
 				{ "long-desc2",
-					"IWantThisDescriptionToBreakInsideAWordGeneratingAutoWordHyphenation.",
+					"IWantThisDescriptionToBreakInsideAWordGeneratingAutoWordHyphenation. ",
 					v => {} },
 				{ "long-desc3",
 					"OnlyOnePeriod.AndNoWhitespaceShouldBeSupportedEvenWithLongDescriptions",
@@ -407,10 +413,14 @@ namespace Tests.Mono.Options
 				{ "long-desc5",
 					"Lots of spaces in the middle - . - . - . - . - . - . - . - and more until the end.",
 					v => {} },
+				"",
+				"==This is a really long category name which will involve line wrapping, just because...==",
 				{ "o|out=",
 					"The {DIRECTORY} to place the generated files and directories.\n\n" +
 					"If not specified, defaults to\n`dirname FILE`/cache/`basename FILE .tree`.",
 					v => {} },
+				"",
+				"Category 3:",
 				{ "h|?|help",           "show help text",                       v => {} },
 				{ "version",            "output version information and exit",  v => {} },
 				{ "<>", v => {} },
@@ -418,6 +428,8 @@ namespace Tests.Mono.Options
 			};
 
 			StringWriter expected = new StringWriter ();
+			expected.WriteLine ("");
+			expected.WriteLine (":Category 1:");
 			expected.WriteLine ("  -p, --indicator-style=VALUE");
 			expected.WriteLine ("                             append / indicator to directories");
 			expected.WriteLine ("      --color[=VALUE]        controls color info");
@@ -441,11 +453,16 @@ namespace Tests.Mono.Options
 			expected.WriteLine ("                               2 3 4 5 and more until the end.");
 			expected.WriteLine ("      --long-desc5           Lots of spaces in the middle - . - . - . - . - . -");
 			expected.WriteLine ("                               . - . - and more until the end.");
+			expected.WriteLine ("");
+			expected.WriteLine ("==This is a really long category name which will involve line wrapping, just");
+			expected.WriteLine ("because...==");
  			expected.WriteLine ("  -o, --out=DIRECTORY        The DIRECTORY to place the generated files and");
 			expected.WriteLine ("                               directories.");
  			expected.WriteLine ("                               ");
 			expected.WriteLine ("                               If not specified, defaults to");
  			expected.WriteLine ("                               `dirname FILE`/cache/`basename FILE .tree`.");
+			expected.WriteLine ("");
+			expected.WriteLine ("Category 3:");
 			expected.WriteLine ("  -h, -?, --help             show help text");
 			expected.WriteLine ("      --version              output version information and exit");
 			expected.WriteLine ("  @s1, @s2                   Read Response File for More Options");
@@ -735,7 +752,7 @@ namespace Tests.Mono.Options
 			Utils.AssertException (typeof(ArgumentException), "prototypes must be null!",
 					p, v => { v.Add ("N|NUM=", (int n) => {}); });
 			Utils.AssertException (typeof(ArgumentNullException),
-					"Argument cannot be null.\nParameter name: option",
+					"Value cannot be null.\nParameter name: option",
 					p, v => { v.GetOptionForName (null); });
 		}
 
@@ -863,6 +880,22 @@ namespace Tests.Mono.Options
 			Assert.AreEqual (formats ["baz"][0], "e");
 			Assert.AreEqual (formats ["baz"][1], "f");
 		}
+
+		[Test]
+		public void ReportInvalidDuplication ()
+		{
+			int verbosity = 0;
+			var p = new OptionSet () {
+				{ "v", v => verbosity = v != null ? verbosity + 1 : verbosity },
+			};
+			try {
+				p.Parse (new []{"-v-v-v"});
+				Assert.Fail ("Should not be reached.");
+			} catch (OptionException e) {
+				Assert.AreEqual (null, e.OptionName);
+				Assert.AreEqual ("Cannot use unregistered option '-' in bundle '-v-v-v'.", e.Message);
+			}
+		}
 	}
 }
 
