@@ -32,19 +32,16 @@ using System.Runtime.CompilerServices;
 using System.Security.Permissions;
 using System.Text.RegularExpressions;
 using System.Web;
+using System.Globalization;
 
 namespace System.Web.Routing
 {
-#if NET_4_0
 	[TypeForwardedFrom ("System.Web.Routing, Version=3.5.0.0, Culture=Neutral, PublicKeyToken=31bf3856ad364e35")]
-#endif
 	[AspNetHostingPermission (SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
 	[AspNetHostingPermission (SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
 	public class Route : RouteBase
 	{
-#if NET_4_0
 		static readonly Type httpRequestBaseType = typeof (HttpRequestBase);
-#endif
 		PatternParser url;
 
 		public RouteValueDictionary Constraints { get; set; }
@@ -102,11 +99,8 @@ namespace System.Web.Routing
 			if (values == null)
 				return null;
 
-			RouteValueDictionary constraints = Constraints;
-			if (constraints != null)
-				foreach (var p in constraints)
-					if (!ProcessConstraint (httpContext, p.Value, p.Key, values, RouteDirection.IncomingRequest))
-						return null;
+			if (!ProcessConstraints (httpContext, values, RouteDirection.IncomingRequest))
+				return null;
 			
 			var rd = new RouteData (this, RouteHandler);
 			RouteValueDictionary rdValues = rd.Values;
@@ -135,14 +129,27 @@ namespace System.Web.Routing
 			// if (values == null)
 			// 	values = requestContext.RouteData.Values;
 
-			string s;
-			if (!url.BuildUrl (this, requestContext, values, out s))
+			RouteValueDictionary usedValues;
+			string resultUrl = url.BuildUrl (this, requestContext, values, Constraints, out usedValues);
+
+			if (resultUrl == null)
 				return null;
 
-			return new VirtualPathData (this, s);
+			if (!ProcessConstraints (requestContext.HttpContext, usedValues, RouteDirection.UrlGeneration))
+				return null;
+
+			var result = new VirtualPathData (this, resultUrl);
+
+			RouteValueDictionary dataTokens = DataTokens;
+			if (dataTokens != null) {
+				foreach (var item in dataTokens)
+					result.DataTokens[item.Key] = item.Value;
+			}
+
+			return result;
 		}
 
-		internal static bool ProcessConstraintInternal (HttpContextBase httpContext, Route route, object constraint, string parameterName,
+		private bool ProcessConstraintInternal (HttpContextBase httpContext, Route route, object constraint, string parameterName,
 								RouteValueDictionary values, RouteDirection routeDirection, RequestContext reqContext,
 								out bool invalidConstraint)
 		{
@@ -153,17 +160,18 @@ namespace System.Web.Routing
 
 			string s = constraint as string;
 			if (s != null) {
-				string v;
+				string v = null;
 				object o;
 
+				// NOTE: If constraint was not an IRouteConstraint, is is asumed
+				// to be an object 'convertible' to string, or at least this is how
+				// ASP.NET seems to work by the tests i've done latelly. (pruiz)
+
 				if (values != null && values.TryGetValue (parameterName, out o))
-					v = o as string;
-				else
-					v = null;
+					v = Convert.ToString (o, CultureInfo.InvariantCulture);
 
 				if (!String.IsNullOrEmpty (v))
 					return MatchConstraintRegex (v, s);
-#if NET_4_0
 				else if (reqContext != null) {
 					RouteData rd = reqContext != null ? reqContext.RouteData : null;
 					RouteValueDictionary rdValues = rd != null ? rd.Values : null;
@@ -174,13 +182,12 @@ namespace System.Web.Routing
 					if (!rdValues.TryGetValue (parameterName, out o))
 						return false;
 
-					v = o as string;
+					v = Convert.ToString (o, CultureInfo.InvariantCulture);
 					if (String.IsNullOrEmpty (v))
 						return false;
 
 					return MatchConstraintRegex (v, s);
 				}
-#endif
 				return false;
 			}
 
@@ -188,22 +195,22 @@ namespace System.Web.Routing
 			return false;
 		}
 
-		static bool MatchConstraintRegex (string rx, string constraint)
+		static bool MatchConstraintRegex (string value, string constraint)
 		{
-			int rxlen = rx.Length;
-			if (rxlen > 0) {
+			int len = constraint.Length;
+			if (len > 0) {
 				// Bug #651966 - regexp constraints must be treated
 				// as absolute expressions
-				if (rx [0] != '^') {
-					rx = "^" + rx;
-					rxlen++;
+				if (constraint [0] != '^') {
+					constraint = "^" + constraint;
+					len++;
 				}
 
-				if (rx [rxlen - 1] != '$')
-					rx += "$";
+				if (constraint [len - 1] != '$')
+					constraint += "$";
 			}
 
-			return Regex.Match (rx, constraint).Success;
+			return Regex.IsMatch (value, constraint, RegexOptions.CultureInvariant | RegexOptions.IgnoreCase | RegexOptions.Compiled);
 		}
 		
 		protected virtual bool ProcessConstraint (HttpContextBase httpContext, object constraint, string parameterName, RouteValueDictionary values, RouteDirection routeDirection)
@@ -216,11 +223,7 @@ namespace System.Web.Routing
 				throw new NullReferenceException ();
 
 			RequestContext reqContext;
-#if NET_4_0
 			reqContext = SafeGetContext (httpContext != null ? httpContext.Request : null);
-#else
-			reqContext = null;
-#endif
 			bool invalidConstraint;
 			bool ret = ProcessConstraintInternal (httpContext, this, constraint, parameterName, values, routeDirection, reqContext, out invalidConstraint);
 			
@@ -234,7 +237,20 @@ namespace System.Web.Routing
 
 			return ret;
 		}
-#if NET_4_0
+
+		private bool ProcessConstraints (HttpContextBase httpContext, RouteValueDictionary values, RouteDirection routeDirection)
+		{
+			var constraints = Constraints;
+
+			if (Constraints != null) {
+				foreach (var p in constraints)
+					if (!ProcessConstraint (httpContext, p.Value, p.Key, values, routeDirection))
+						return false;
+			}
+
+			return true;
+		}
+
 		RequestContext SafeGetContext (HttpRequestBase req)
 		{
 			if (req == null || req.GetType () != httpRequestBaseType)
@@ -242,6 +258,5 @@ namespace System.Web.Routing
 				
 			return req.RequestContext;
 		}
-#endif
 	}
 }
