@@ -16,6 +16,8 @@ using System.Text;
 using System.Threading;
 using NUnit.Framework;
 
+using MonoTests.Helpers;
+
 namespace MonoTests.System.Net
 {
 	[TestFixture]
@@ -63,7 +65,6 @@ namespace MonoTests.System.Net
 			try {
 				wc.DownloadData ((string) null);
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -71,24 +72,6 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("address", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("uriString", inner.ParamName, "#11");
-			}
-#endif
 		}
 
 		[Test] // DownloadData (string)
@@ -114,7 +97,6 @@ namespace MonoTests.System.Net
 			}
 		}
 
-#if NET_2_0
 		[Test] // DownloadData (Uri)
 		public void DownloadData2_Address_Null ()
 		{
@@ -153,7 +135,6 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (inner.Message, "#9");
 			}
 		}
-#endif
 
 		[Test]
 		public void DownloadFile1_Address_Null ()
@@ -162,7 +143,6 @@ namespace MonoTests.System.Net
 			try {
 				wc.DownloadFile ((string) null, "tmp.out");
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -170,24 +150,6 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("address", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("uriString", inner.ParamName, "#11");
-			}
-#endif
 		}
 
 		[Test] // DownloadFile (string, string)
@@ -226,7 +188,6 @@ namespace MonoTests.System.Net
 				wc.DownloadFile ("tp://scheme.notsupported",
 					(string) null);
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -234,27 +195,8 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("fileName", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("path", inner.ParamName, "#11");
-			}
-#endif
 		}
 
-#if NET_2_0
 		[Test] // DownloadFile (Uri, string)
 		public void DownloadFile2_Address_Null ()
 		{
@@ -417,7 +359,6 @@ namespace MonoTests.System.Net
 				Assert.AreEqual ("Encoding", ex.ParamName, "#6");
 			}
 		}
-#endif
 
 		[Test] // OpenRead (string)
 		public void OpenRead1_Address_Null ()
@@ -426,7 +367,6 @@ namespace MonoTests.System.Net
 			try {
 				wc.OpenRead ((string) null);
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -434,24 +374,6 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("address", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("uriString", inner.ParamName, "#11");
-			}
-#endif
 		}
 
 		[Test] // OpenRead (string)
@@ -477,7 +399,6 @@ namespace MonoTests.System.Net
 			}
 		}
 
-#if NET_2_0
 		[Test] // OpenRead (Uri)
 		public void OpenRead2_Address_Null ()
 		{
@@ -516,7 +437,6 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (inner.Message, "#9");
 			}
 		}
-#endif
 
 		[Test] // OpenWrite (string)
 		public void OpenWrite1_Address_Null ()
@@ -525,7 +445,6 @@ namespace MonoTests.System.Net
 			try {
 				wc.OpenWrite ((string) null);
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -533,24 +452,6 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("address", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("uriString", inner.ParamName, "#11");
-			}
-#endif
 		}
 
 		[Test] // OpenWrite (string)
@@ -583,7 +484,6 @@ namespace MonoTests.System.Net
 			try {
 				wc.OpenWrite ((string) null, "PUT");
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -591,24 +491,6 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("address", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("uriString", inner.ParamName, "#11");
-			}
-#endif
 		}
 
 		[Test] // OpenWrite (string, string)
@@ -634,7 +516,6 @@ namespace MonoTests.System.Net
 			}
 		}
 
-#if NET_2_0
 		[Test] // OpenWrite (Uri)
 		public void OpenWrite3_Address_Null ()
 		{
@@ -713,7 +594,6 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (inner.Message, "#9");
 			}
 		}
-#endif
 
 		[Test] // UploadData (string, byte [])
 		public void UploadData1_Address_Null ()
@@ -722,7 +602,6 @@ namespace MonoTests.System.Net
 			try {
 				wc.UploadData ((string) null, new byte [] { 0x1a });
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -730,24 +609,6 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("address", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("uriString", inner.ParamName, "#11");
-			}
-#endif
 		}
 
 		[Test] // UploadData (string, byte [])
@@ -774,9 +635,6 @@ namespace MonoTests.System.Net
 		}
 
 		[Test] // UploadData (string, byte [])
-#if ONLY_1_1
-		[Category ("NotDotNet")] // On MS, there's a nested NotImplementedException
-#endif
 		public void UploadData1_Data_Null ()
 		{
 			WebClient wc = new WebClient ();
@@ -784,7 +642,6 @@ namespace MonoTests.System.Net
 				wc.UploadData ("http://www.mono-project.com",
 					(byte []) null);
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -792,27 +649,8 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("data", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("data", inner.ParamName, "#11");
-			}
-#endif
 		}
 
-#if NET_2_0
 		[Test] // UploadData (Uri, byte [])
 		public void UploadData2_Address_Null ()
 		{
@@ -869,7 +707,6 @@ namespace MonoTests.System.Net
 				Assert.AreEqual ("data", ex.ParamName, "#6");
 			}
 		}
-#endif
 
 		[Test] // UploadData (string, string, byte [])
 		public void UploadData3_Address_Null ()
@@ -879,7 +716,6 @@ namespace MonoTests.System.Net
 				wc.UploadData ((string) null, "POST",
 					new byte [] { 0x1a });
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -887,24 +723,6 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("address", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("uriString", inner.ParamName, "#11");
-			}
-#endif
 		}
 
 		[Test] // UploadData (string, string, byte [])
@@ -932,9 +750,6 @@ namespace MonoTests.System.Net
 		}
 
 		[Test] // UploadData (string, string, byte [])
-#if ONLY_1_1
-		[Category ("NotDotNet")] // On MS, there's a nested NotImplementedException
-#endif
 		public void UploadData3_Data_Null ()
 		{
 			WebClient wc = new WebClient ();
@@ -942,7 +757,6 @@ namespace MonoTests.System.Net
 				wc.UploadData ("http://www.mono-project.com",
 					"POST", (byte []) null);
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -950,27 +764,8 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("data", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("data", inner.ParamName, "#11");
-			}
-#endif
 		}
 
-#if NET_2_0
 		[Test] // UploadData (Uri, string, byte [])
 		public void UploadData4_Address_Null ()
 		{
@@ -1027,7 +822,6 @@ namespace MonoTests.System.Net
 				Assert.AreEqual ("data", ex.ParamName, "#6");
 			}
 		}
-#endif
 
 		[Test] // UploadFile (string, string)
 		public void UploadFile1_Address_Null ()
@@ -1039,7 +833,6 @@ namespace MonoTests.System.Net
 			try {
 				wc.UploadFile ((string) null, tempFile);
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -1047,24 +840,6 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("address", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("uriString", inner.ParamName, "#11");
-			}
-#endif
 		}
 
 		[Test] // UploadFile (string, string)
@@ -1131,7 +906,6 @@ namespace MonoTests.System.Net
 				wc.UploadFile ("tp://scheme.notsupported",
 					(string) null);
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -1139,27 +913,8 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("fileName", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("path", inner.ParamName, "#11");
-			}
-#endif
 		}
 
-#if NET_2_0
 		[Test] // UploadFile (Uri, string)
 		public void UploadFile2_Address_Null ()
 		{
@@ -1250,7 +1005,6 @@ namespace MonoTests.System.Net
 				Assert.AreEqual ("fileName", ex.ParamName, "#6");
 			}
 		}
-#endif
 
 		[Test] // UploadFile (string, string, string)
 		public void UploadFile3_Address_Null ()
@@ -1262,7 +1016,6 @@ namespace MonoTests.System.Net
 			try {
 				wc.UploadFile ((string) null, "POST", tempFile);
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -1270,24 +1023,6 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("path", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("uriString", inner.ParamName, "#11");
-			}
-#endif
 		}
 
 		[Test] // UploadFile (string, string, string)
@@ -1354,7 +1089,6 @@ namespace MonoTests.System.Net
 				wc.UploadFile ("tp://scheme.notsupported",
 					"POST", (string) null);
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -1362,27 +1096,8 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("fileName", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("path", inner.ParamName, "#11");
-			}
-#endif
 		}
 
-#if NET_2_0
 		[Test] // UploadFile (Uri, string, string)
 		public void UploadFile4_Address_Null ()
 		{
@@ -1700,21 +1415,19 @@ namespace MonoTests.System.Net
 				Assert.AreEqual ("data", ex.ParamName, "#6");
 			}
 		}
-#endif
 
 		[Test]
+		[Category ("AndroidNotWorking")] // Fails when ran as part of the entire BCL test suite. Works when only this fixture is ran
 		public void UploadValues1 ()
 		{
-			IPEndPoint ep = new IPEndPoint (IPAddress.Loopback, 8000);
-			string url = "http://" + IPAddress.Loopback.ToString () + ":8000/test/";
+			IPEndPoint ep = NetworkHelpers.LocalEphemeralEndPoint ();
+			string url = "http://" + ep.ToString () + "/test/";
 
 			using (SocketResponder responder = new SocketResponder (ep, new SocketRequestHandler (EchoRequestHandler))) {
 				responder.Start ();
 
 				WebClient wc = new WebClient ();
-#if NET_2_0
 				wc.Encoding = Encoding.ASCII;
-#endif
 
 				NameValueCollection nvc = new NameValueCollection ();
 				nvc.Add ("Name", "\u0041\u2262\u0391\u002E");
@@ -1733,7 +1446,6 @@ namespace MonoTests.System.Net
 			try {
 				wc.UploadValues ((string) null, new NameValueCollection ());
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -1741,24 +1453,6 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("address", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("uriString", inner.ParamName, "#11");
-			}
-#endif
 		}
 
 		[Test] // UploadValues (string, NameValueCollection)
@@ -1786,9 +1480,6 @@ namespace MonoTests.System.Net
 		}
 
 		[Test] // UploadValues (string, NameValueCollection)
-#if ONLY_1_1
-		[Category ("NotDotNet")] // On MS, there's a nested NotImplementedException
-#endif
 		public void UploadValues1_Data_Null ()
 		{
 			WebClient wc = new WebClient ();
@@ -1796,7 +1487,6 @@ namespace MonoTests.System.Net
 				wc.UploadValues ("http://www.mono-project.com",
 					(NameValueCollection) null);
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -1804,27 +1494,8 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("data", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("data", inner.ParamName, "#11");
-			}
-#endif
 		}
 
-#if NET_2_0
 		[Test] // UploadValues (Uri, NameValueCollection)
 		public void UploadValues2_Address_Null ()
 		{
@@ -1881,7 +1552,6 @@ namespace MonoTests.System.Net
 				Assert.AreEqual ("data", ex.ParamName, "#6");
 			}
 		}
-#endif
 
 		[Test] // UploadValues (string, string, NameValueCollection)
 		public void UploadValues3_Address_Null ()
@@ -1891,7 +1561,6 @@ namespace MonoTests.System.Net
 				wc.UploadValues ((string) null, "POST",
 					new NameValueCollection ());
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -1899,24 +1568,6 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("address", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("uriString", inner.ParamName, "#11");
-			}
-#endif
 		}
 
 		[Test] // UploadValues (string, string, NameValueCollection)
@@ -1944,9 +1595,6 @@ namespace MonoTests.System.Net
 		}
 
 		[Test] // UploadValues (string, string, NameValueCollection)
-#if ONLY_1_1
-		[Category ("NotDotNet")] // On MS, there's a nested NotImplementedException
-#endif
 		public void UploadValues3_Data_Null ()
 		{
 			WebClient wc = new WebClient ();
@@ -1954,7 +1602,6 @@ namespace MonoTests.System.Net
 				wc.UploadValues ("http://www.mono-project.com",
 					"POST", (NameValueCollection) null);
 				Assert.Fail ("#1");
-#if NET_2_0
 			} catch (ArgumentNullException ex) {
 				Assert.AreEqual (typeof (ArgumentNullException), ex.GetType (), "#2");
 				Assert.IsNull (ex.InnerException, "#3");
@@ -1962,27 +1609,8 @@ namespace MonoTests.System.Net
 				Assert.IsNotNull (ex.ParamName, "#5");
 				Assert.AreEqual ("data", ex.ParamName, "#6");
 			}
-#else
-			} catch (WebException ex) {
-				// An error occurred performing a WebClient request
-				Assert.AreEqual (typeof (WebException), ex.GetType (), "#2");
-				Assert.IsNotNull (ex.Message, "#3");
-				Assert.IsNull (ex.Response, "#4");
-				Assert.AreEqual (WebExceptionStatus.UnknownError, ex.Status, "#5");
-
-				ArgumentNullException inner = ex.InnerException
-					as ArgumentNullException;
-				Assert.IsNotNull (inner, "#6");
-				Assert.AreEqual (typeof (ArgumentNullException), inner.GetType (), "#7");
-				Assert.IsNull (inner.InnerException, "#8");
-				Assert.IsNotNull (inner.Message, "#9");
-				Assert.IsNotNull (inner.ParamName, "#10");
-				Assert.AreEqual ("data", inner.ParamName, "#11");
-			}
-#endif
 		}
 
-#if NET_2_0
 		[Test] // UploadValues (Uri, string, NameValueCollection)
 		public void UploadValues4_Address_Null ()
 		{
@@ -2040,9 +1668,7 @@ namespace MonoTests.System.Net
 				Assert.AreEqual ("data", ex.ParamName, "#6");
 			}
 		}
-#endif
 
-#if NET_2_0
 		[Test]
 		public void GetWebRequestOverriding ()
 		{
@@ -2061,7 +1687,6 @@ namespace MonoTests.System.Net
 				return base.GetWebRequest (address);
 			}
 		}
-#endif
 
 		static byte [] EchoRequestHandler (Socket socket)
 		{
@@ -2159,9 +1784,10 @@ namespace MonoTests.System.Net
 		 
 #if NET_4_5
 		[Test]
+		[Category ("AndroidNotWorking")] // Fails when ran as part of the entire BCL test suite. Works when only this fixture is ran
 		public void UploadStringAsyncCancelEvent ()
 		{
-			UploadAsyncCancelEventTest ((webClient, uri, cancelEvent) =>
+			UploadAsyncCancelEventTest (9301, (webClient, uri, cancelEvent) =>
 			{
 
 				webClient.UploadStringCompleted += (sender, args) =>
@@ -2175,9 +1801,10 @@ namespace MonoTests.System.Net
 		}
 
 		[Test]
+		[Category ("AndroidNotWorking")] // Fails when ran as part of the entire BCL test suite. Works when only this fixture is ran
 		public void UploadDataAsyncCancelEvent ()
 		{
-			UploadAsyncCancelEventTest ((webClient, uri, cancelEvent) =>
+			UploadAsyncCancelEventTest (9302, (webClient, uri, cancelEvent) =>
 			{
 				webClient.UploadDataCompleted += (sender, args) =>
 				{
@@ -2190,9 +1817,10 @@ namespace MonoTests.System.Net
 		}
 		
 		[Test]
+		[Category ("AndroidNotWorking")] // Fails when ran as part of the entire BCL test suite. Works when only this fixture is ran
 		public void UploadValuesAsyncCancelEvent ()
 		{
-			UploadAsyncCancelEventTest ((webClient, uri, cancelEvent) =>
+			UploadAsyncCancelEventTest (9303, (webClient, uri, cancelEvent) =>
 			{
 				webClient.UploadValuesCompleted += (sender, args) =>
 				{
@@ -2205,9 +1833,10 @@ namespace MonoTests.System.Net
 		}
 
 		[Test]
+		[Category ("AndroidNotWorking")] // Fails when ran as part of the entire BCL test suite. Works when only this fixture is ran
 		public void UploadFileAsyncCancelEvent ()
 		{
-			UploadAsyncCancelEventTest ((webClient, uri, cancelEvent) =>
+			UploadAsyncCancelEventTest (9304,(webClient, uri, cancelEvent) =>
 			{
 				string tempFile = Path.Combine (_tempFolder, "upload.tmp");
 				File.Create (tempFile).Close ();
@@ -2221,13 +1850,36 @@ namespace MonoTests.System.Net
 				webClient.UploadFileAsync (uri, "PUT", tempFile);
 			});
 		}
+
+		[Test]
+		[Category ("AndroidNotWorking")] // Test suite hangs if the tests runs as part of the entire BCL suite. Works when only this fixture is ran
+		public void UploadFileAsyncContentType ()
+		{
+			var serverUri = "http://localhost:13370/";
+			var filename = Path.GetTempFileName ();
+
+			HttpListener listener = new HttpListener ();
+			listener.Prefixes.Add (serverUri);
+			listener.Start ();
+
+			using (var client = new WebClient ())
+			{
+				client.UploadFileTaskAsync (new Uri (serverUri), filename);
+				var request = listener.GetContext ().Request;
+
+				var expected = "multipart/form-data; boundary=---------------------";
+				Assert.AreEqual (expected.Length + 15, request.ContentType.Length);
+				Assert.AreEqual (expected, request.ContentType.Substring (0, expected.Length));
+			}
+			listener.Close ();
+		}
 #endif
 
 #if NET_4_0
-		public void UploadAsyncCancelEventTest (Action<WebClient, Uri, EventWaitHandle> uploadAction)
+		public void UploadAsyncCancelEventTest (int port, Action<WebClient, Uri, EventWaitHandle> uploadAction)
 		{
-			var ep = new IPEndPoint (IPAddress.Loopback, 8000);
-			string url = "http://" + IPAddress.Loopback + ":8000/test/";
+			var ep = NetworkHelpers.LocalEphemeralEndPoint ();
+			string url = "http://" + ep.ToString() + "/test/";
 
 			using (var responder = new SocketResponder (ep, EchoRequestHandler))
 			{
