@@ -33,15 +33,14 @@ using System.Runtime.CompilerServices;
 using System.Runtime.InteropServices;
 using System.Runtime.Serialization;
 using System.Security.Permissions;
+using System.Security.Claims;
 
 namespace System.Security.Principal {
 
 	[Serializable]
 	[ComVisible (true)]
 	public class WindowsIdentity :
-#if NET_4_5
 	System.Security.Claims.ClaimsIdentity,
-#endif
 	IIdentity, IDeserializationCallback, ISerializable, IDisposable {
 		private IntPtr _token;
 		private string _type;
@@ -52,10 +51,8 @@ namespace System.Security.Principal {
 
 		static private IntPtr invalidWindows = IntPtr.Zero;
 
-#if NET_4_5
 		[NonSerialized]
 		public new const string DefaultIssuer = "AD AUTHORITY";
-#endif
 
 		[SecurityPermission (SecurityAction.Demand, ControlPrincipal=true)]
 		public WindowsIdentity (IntPtr userToken) 
@@ -117,6 +114,15 @@ namespace System.Security.Principal {
 			_info = info;
 		}
 
+		internal WindowsIdentity (ClaimsIdentity claimsIdentity, IntPtr userToken)
+			: base (claimsIdentity)
+		{
+			if (userToken != IntPtr.Zero && userToken.ToInt64() > 0)
+			{
+				SetToken (userToken);
+			}
+		}
+
 		[ComVisible (false)]
 		public void Dispose ()
 		{
@@ -177,9 +183,7 @@ namespace System.Security.Principal {
 		}
 
 		// properties
-#if NET_4_5
 		sealed override
-#endif
 		public string AuthenticationType {
 			get { return _type; }
 		}
@@ -189,11 +193,7 @@ namespace System.Security.Principal {
 			get { return (_account == WindowsAccountType.Anonymous); }
 		}
 
-#if NET_4_5
 		override
-#else
-		virtual
-#endif
 		public bool IsAuthenticated
 		{
 			get { return _authenticated; }
@@ -209,11 +209,7 @@ namespace System.Security.Principal {
 			get { return (_account == WindowsAccountType.System); }
 		}
 
-#if NET_4_5
 		override
-#else
-		virtual
-#endif
 		public string Name
 		{
 			get {
@@ -283,6 +279,16 @@ namespace System.Security.Principal {
 			info.AddValue ("m_isAuthenticated", _authenticated);
 		}
 
+		internal ClaimsIdentity CloneAsBase ()
+		{
+			return base.Clone();
+		}
+
+		internal IntPtr GetTokenInternal ()
+		{
+			return _token;
+		}
+
 		private void SetToken (IntPtr token) 
 		{
 			if (Environment.IsUnix) {
