@@ -8,7 +8,7 @@
 //
 
 using System;
-using System.IO;
+using System.Reflection;
 
 using Mono.Cecil;
 
@@ -16,7 +16,7 @@ namespace Mono.CilStripper {
 
 	class Program {
 
-		static void Main (string [] args)
+		static int Main (string [] args)
 		{
 			Header ();
 
@@ -34,18 +34,18 @@ namespace Mono.CilStripper {
 					Console.WriteLine ("Assembly {0} stripped out into {1}", file, output);
 				else
 					Console.WriteLine ("Assembly {0} stripped", file);
+				return 0;
+			} catch (TargetInvocationException tie) {
+				Console.WriteLine ("Error: {0}", tie.InnerException);
 			} catch (Exception e) {
 				Console.WriteLine ("Error: {0}", e);
 			}
+			return 1;
 		}
 
 		static void StripAssembly (AssemblyDefinition assembly, string output)
 		{
-			Type stripper = typeof (AssemblyDefinition).Assembly.GetType ("Mono.Cecil.AssemblyStripper");
-			if (stripper == null)
-				throw new NotSupportedException ("Cecil doesn't have support for mono-cil-strip");
-
-			stripper.GetMethod ("StripAssembly").Invoke (null, new object [] { assembly, output });
+			AssemblyStripper.StripAssembly (assembly, output);
 		}
 
 		static void Header ()
