@@ -13,7 +13,6 @@
 #include <config.h>
 #include <glib.h>
 #include <errno.h>
-#include <signal.h>
 #include <string.h>
 #include <sys/types.h>
 
@@ -21,6 +20,7 @@
 #include <mono/io-layer/misc-private.h>
 #include <mono/io-layer/collection.h>
 #include <mono/io-layer/shared.h>
+#include <mono/utils/atomic.h>
 
 #define _WAPI_PRIVATE_MAX_SLOTS		(1024 * 16)
 #define _WAPI_PRIVATE_HANDLES(x) (_wapi_private_handles [x / _WAPI_HANDLE_INITIAL_COUNT][x % _WAPI_HANDLE_INITIAL_COUNT])
@@ -68,7 +68,8 @@ extern void _wapi_handle_ops_signal (gpointer handle);
 extern gboolean _wapi_handle_ops_own (gpointer handle);
 extern gboolean _wapi_handle_ops_isowned (gpointer handle);
 extern guint32 _wapi_handle_ops_special_wait (gpointer handle,
-					      guint32 timeout);
+					      guint32 timeout,
+					      gboolean alertable);
 extern void _wapi_handle_ops_prewait (gpointer handle);
 
 extern gboolean _wapi_handle_count_signalled_handles (guint32 numhandles,
@@ -83,7 +84,7 @@ extern int _wapi_handle_timedwait_signal (struct timespec *timeout, gboolean pol
 extern int _wapi_handle_wait_signal_handle (gpointer handle, gboolean alertable);
 extern int _wapi_handle_timedwait_signal_handle (gpointer handle,
 												 struct timespec *timeout, gboolean alertable, gboolean poll);
-extern gboolean _wapi_handle_get_or_set_share (dev_t device, ino_t inode,
+extern gboolean _wapi_handle_get_or_set_share (guint64 device, guint64 inode,
 					       guint32 new_sharemode,
 					       guint32 new_access,
 					       guint32 *old_sharemode,
@@ -109,8 +110,8 @@ static inline WapiHandleType _wapi_handle_type (gpointer handle)
 {
 	guint32 idx = GPOINTER_TO_UINT(handle);
 	
-	if (!_WAPI_PRIVATE_VALID_SLOT (idx)) {
-		return(WAPI_HANDLE_COUNT);	/* An impossible type */
+	if (!_WAPI_PRIVATE_VALID_SLOT (idx) || !_WAPI_PRIVATE_HAVE_SLOT (idx)) {
+		return(WAPI_HANDLE_UNUSED);	/* An impossible type */
 	}
 	
 	return(_WAPI_PRIVATE_HANDLES(idx).type);
@@ -143,7 +144,6 @@ static inline void _wapi_handle_set_signal_state (gpointer handle,
 		/* The condition the global signal cond is waiting on is the signalling of
 		 * _any_ handle. So lock it before setting the signalled state.
 		 */
-		pthread_cleanup_push ((void(*)(void *))mono_mutex_unlock_in_cleanup, (void *)_wapi_global_signal_mutex);
 		thr_ret = mono_mutex_lock (_wapi_global_signal_mutex);
 		if (thr_ret != 0)
 			g_warning ("Bad call to mono_mutex_lock result %d for global signal mutex", thr_ret);
@@ -178,8 +178,6 @@ static inline void _wapi_handle_set_signal_state (gpointer handle,
 		if (thr_ret != 0)
 			g_warning ("Bad call to mono_mutex_unlock result %d for global signal mutex", thr_ret);
 		g_assert (thr_ret == 0);
-
-		pthread_cleanup_pop (0);
 	} else {
 		handle_data->signalled=state;
 	}
